@@ -47,6 +47,7 @@
 #include "global.h"
 
 #include "main.h"
+#include "log.h"
 #include "support.h"
 #include "gui_support.h"
 #include "filer.h"
@@ -71,6 +72,7 @@
 #include "session.h"
 #include "minibuffer.h"
 #include "xtypes.h"
+#include "bulk_rename.h"
 
 int number_of_windows = 0;	/* Quit when this reaches 0 again... */
 int to_wakeup_pipe = -1;	/* Write here to get noticed */
@@ -129,12 +131,13 @@ GtkTooltips *tooltips = NULL;
        "  -S, --rox-session	use default panel and pinboard options, and -n\n"\
        "  -t, --top=PANEL	open PANEL as a top-edge panel\n"	\
        "  -u, --user		show user name in each window \n"	\
+       "  -U, --url=URL		open file or directory in URI form\n"   \
        "  -v, --version		display the version information and exit\n"   \
        "  -x, --examine=FILE	FILE has changed - re-examine it\n"	\
        "\nReport bugs to %s.\n"		\
        "Home page (including updated versions): http://rox.sourceforge.net/\n")
 
-#define SHORT_OPS "c:d:t:b:l:r:B:op:s:hvnux:m:D:RS"
+#define SHORT_OPS "c:d:t:b:l:r:B:op:s:hvnux:m:D:RSU:"
 
 #ifdef HAVE_GETOPT_LONG
 static struct option long_opts[] =
@@ -158,6 +161,7 @@ static struct option long_opts[] =
 	{"close", 1, NULL, 'D'},
 	{"mime-type", 1, NULL, 'm'},
 	{"client-id", 1, NULL, 'c'},
+	{"url", 1, NULL, 'u'},
 	{NULL, 0, NULL, 0},
 };
 #endif
@@ -172,7 +176,7 @@ enum {
 	SESSION_BOTH,
 };
 Option o_session_panel_or_pin;
-Option o_session_panel_name;
+Option o_session_panel_name;	/* Now a comma-separated list */
 Option o_session_pinboard_name;
 
 /* Always start a new filer, even if one seems to be already running */
@@ -190,6 +194,7 @@ static void soap_add(xmlNodePtr body,
 			   xmlChar *function,
 			   const xmlChar *arg1_name, const xmlChar *arg1_value,
 			   const xmlChar *arg2_name, const xmlChar *arg2_value);
+static void soap_reply(xmlDocPtr reply, gboolean rpc_mode);
 static void child_died(int signum);
 static void child_died_callback(void);
 static void wake_up_cb(gpointer data, gint source, GdkInputCondition condition);
@@ -252,6 +257,7 @@ int main(int argc, char **argv)
 	guchar		*tmp, *dir;
 	gchar *client_id = NULL;
 	gboolean	show_user = FALSE;
+	gboolean	rpc_mode = FALSE;
 	xmlDocPtr	rpc, soap_rpc = NULL, reply;
 	xmlNodePtr	body;
 	int		fd, ofd0=-1;
@@ -336,6 +342,10 @@ int main(int argc, char **argv)
 			  "Default");
 	option_register_widget("launch", build_launch);
 
+#ifdef UNIT_TESTS
+	bulk_rename_tests();
+#endif
+
 	/* The idea here is to convert the command-line arguments
 	 * into a SOAP RPC.
 	 * We attempt to invoke the call on an already-running copy of
@@ -491,12 +501,20 @@ int main(int argc, char **argv)
 					dup2(fd, 0);
 					close(fd);
 				}
+				/* Want to print return uninterpreted */
+				rpc_mode=TRUE;
 				
 				break;
 
-		        case 'S':
+			case 'S':
 				new_copy = TRUE;
 				add_default_panel_and_pinboard(body);
+				session_auto_respawn = TRUE;
+				break;
+
+		        case 'U':
+				soap_add(body, "RunURI",
+						"URI", VALUE, NULL, NULL);
 				break;
 
 			default:
@@ -571,6 +589,7 @@ int main(int argc, char **argv)
 
 	pixmaps_init();
 
+	log_init();
 	dnd_init();
 	bind_init();
 	dir_init();
@@ -626,12 +645,7 @@ int main(int argc, char **argv)
 	/* Finally, execute the request */
 	reply = run_soap(rpc);
 	xmlFreeDoc(rpc);
-	if (reply)
-	{
-		/* Write the result, if any, to stdout */
-		save_xml_file(reply, "-");
-		xmlFreeDoc(reply);
-	}
+	soap_reply(reply, rpc_mode);
 
 	/* Try to find out why we crash with GTK 2.4 */
 	XSetErrorHandler(rox_x_error);
@@ -686,6 +700,13 @@ static void show_features(void)
 		_("No")
 #endif
 		);
+	g_print("%s... %s\n", _("Inotify support"),
+#ifdef USE_INOTIFY
+		_("Yes")
+#else
+		_("No")
+#endif
+	       );
 	g_print("%s... %s\n", _("Dnotify support"),
 #ifdef USE_DNOTIFY
 		_("Yes")
@@ -725,6 +746,34 @@ static void soap_add(xmlNodePtr body,
 	}
 }
 
+static void soap_reply(xmlDocPtr reply, gboolean rpc_mode)
+{
+	gboolean print=TRUE;
+
+	if(!reply)
+		return;
+
+	if(!rpc_mode) {
+		gchar **errs=extract_soap_errors(reply);
+
+		if(errs) {
+			int i;
+			
+			print=FALSE;
+
+			for(i=0; errs[i]; i++)
+				fprintf(stderr, "%s\n", errs[i]);
+
+			g_strfreev(errs);
+		}
+	}
+
+	/* Write the result, if any, to stdout */
+	if(print)
+		save_xml_file(reply, "-");
+	xmlFreeDoc(reply);
+}
+
 /* This is called as a signal handler; simply ensures that
  * child_died_callback() will get called later.
  */
@@ -791,7 +840,7 @@ static void xrandr_size_change(GdkScreen *screen, gpointer user_data)
 
 static void add_default_panel_and_pinboard(xmlNodePtr body)
 {
-	char *name;
+	const char *name;
 
 	if (o_session_panel_or_pin.int_value != SESSION_PANEL_ONLY)
 	{
@@ -803,15 +852,46 @@ static void add_default_panel_and_pinboard(xmlNodePtr body)
 					
 	if (o_session_panel_or_pin.int_value != SESSION_PINBOARD_ONLY)
 	{
-		name = o_session_panel_name.value;
-		if (!name[0])
-			name="Default";
-		
-		soap_add(body, "Panel", "Name", name, NULL, NULL);
+		gboolean use_old_option = TRUE;
+		GIOChannel *fp = NULL;
+		GError *err = NULL;
+		char *line = NULL;
+		gsize term;
+		char *filename = choices_find_xdg_path_load("panels",
+				"ROX-Filer", "rox.sourceforge.net");
+
+		if (filename)
+			fp = g_io_channel_new_file(filename, "r", &err);
+		while (fp && g_io_channel_read_line(fp, &line, NULL, &term, &err) ==
+				G_IO_STATUS_NORMAL)
+		{
+			if (line && (line[term] = 0, line[0]))
+			{
+				soap_add(body, "Panel", "Name", line, NULL, NULL);
+				use_old_option = FALSE;
+			}
+		}
+		if (err)
+		{
+			g_critical(_("Unable to read '%s': %s"),
+					filename, err->message);
+			g_error_free(err);
+		}
+		if (fp)
+			g_io_channel_shutdown(fp, FALSE, NULL);
+		if (use_old_option)
+		{
+			name = o_session_panel_name.value;
+			if (!name[0])
+				name="Default";
+			soap_add(body, "Panel", "Name", name, NULL, NULL);
+		}
+		g_free(filename);
 	}
 }
 
-static GtkWidget *launch_button_new(const char *label, const char *uri)
+static GtkWidget *launch_button_new(const char *label, const char *uri,
+				    const char *appname)
 {
 	GtkWidget *button;
 	GClosure *closure;
@@ -819,11 +899,16 @@ static GtkWidget *launch_button_new(const char *label, const char *uri)
 	gchar *tip;
 
 	button = button_new_mixed(GTK_STOCK_PREFERENCES, label);
-	closure = g_cclosure_new_swap(G_CALLBACK(launch_uri),
+	closure = g_cclosure_new(G_CALLBACK(launch_uri),
 					g_strdup(uri),
 					(GClosureNotify) g_free);
 	g_signal_connect_closure(button, "clicked", closure, FALSE);
-
+	if(appname) {
+		g_object_set_data_full(G_OBJECT(button), "appname",
+				       g_strdup(appname),
+				       (GDestroyNotify) g_free);
+	}
+	
 	allow_right_click(button);
 
 	slash = strrchr(uri, '/');
@@ -845,20 +930,24 @@ static GList *build_launch(Option *option, xmlNode *node, guchar *label)
 {
 	GtkWidget *align;
 	char *uri;
+	char *appname;
 
 	g_return_val_if_fail(option == NULL, NULL);
 	g_return_val_if_fail(label != NULL, NULL);
 
 	uri = xmlGetProp(node, "uri");
+	appname = xmlGetProp(node, "appname");
 
 	g_return_val_if_fail(uri != NULL, NULL);
 
 	align = gtk_alignment_new(0, 0.5, 0, 0);
 
 	gtk_container_add(GTK_CONTAINER(align),
-			launch_button_new(_(label), uri));
+			  launch_button_new(_(label), uri, appname));
 
 	g_free(uri);
+	if(appname)
+	  g_free(appname);
 
 	return g_list_append(NULL, align);
 }
