@@ -19,6 +19,10 @@
 
 /* menu.c - code for handling the popup menus */
 
+#ifndef GTK_STOCK_INFO
+# define GTK_STOCK_INFO "gtk-info"
+#endif
+
 #include "config.h"
 
 #include <stdio.h>
@@ -61,6 +65,7 @@
 #include "panel.h"
 #include "bulk_rename.h"
 #include "xtypes.h"
+#include "log.h"
 
 typedef enum {
 	FILE_COPY_ITEM,
@@ -127,6 +132,7 @@ static void set_with(gpointer data, guint action, GtkWidget *widget);
 static void set_sort(gpointer data, guint action, GtkWidget *widget);
 static void reverse_sort(gpointer data, guint action, GtkWidget *widget);
 
+static void filter_directories(gpointer data, guint action, GtkWidget *widget);
 static void hidden(gpointer data, guint action, GtkWidget *widget);
 static void show_thumbs(gpointer data, guint action, GtkWidget *widget);
 static void refresh(gpointer data, guint action, GtkWidget *widget);
@@ -146,6 +152,7 @@ static void open_parent_same(gpointer data, guint action, GtkWidget *widget);
 static void open_parent(gpointer data, guint action, GtkWidget *widget);
 static void home_directory(gpointer data, guint action, GtkWidget *widget);
 static void show_bookmarks(gpointer data, guint action, GtkWidget *widget);
+static void show_log(gpointer data, guint action, GtkWidget *widget);
 static void new_window(gpointer data, guint action, GtkWidget *widget);
 /* static void new_user(gpointer data, guint action, GtkWidget *widget); */
 static void close_window(gpointer data, guint action, GtkWidget *widget);
@@ -163,6 +170,7 @@ static GtkWidget	*filer_file_menu;	/* The File '' menu */
 static GtkWidget	*file_shift_item;	/* Shift Open label */
 static GtkWidget	*filer_auto_size_menu;	/* The Automatic item */
 static GtkWidget	*filer_hidden_menu;	/* The Show Hidden item */
+static GtkWidget	*filer_filter_dirs_menu;/* The Filter Dirs item */
 static GtkWidget	*filer_reverse_menu;	/* The Reversed item */
 static GtkWidget	*filer_thumb_menu;	/* The Show Thumbs item */
 static GtkWidget	*filer_new_window;	/* The New Window item */
@@ -197,6 +205,7 @@ static GtkItemFactoryEntry filer_menu_def[] = {
 {">",				NULL, NULL, 0, "<Separator>"},
 {">" N_("Show Hidden"),   	"<Ctrl>H", hidden, 0, "<ToggleItem>"},
 {">" N_("Filter Files..."),   	NULL, mini_buffer, MINI_FILTER, NULL},
+{">" N_("Filter Directories With Files"),	NULL, filter_directories, 0, "<ToggleItem>"},
 {">" N_("Show Thumbnails"),	NULL, show_thumbs, 0, "<ToggleItem>"},
 {">" N_("Refresh"),		NULL, refresh, 0, "<StockItem>", GTK_STOCK_REFRESH},
 {">" N_("Save Current Display Settings..."),	 NULL, save_settings, 0, NULL},
@@ -234,6 +243,7 @@ static GtkItemFactoryEntry filer_menu_def[] = {
 {">" N_("New Window"),		NULL, new_window, 0, NULL},
 {">" N_("Home Directory"),	"<Ctrl>Home", home_directory, 0, "<StockItem>", GTK_STOCK_HOME},
 {">" N_("Show Bookmarks"),	"<Ctrl>B", show_bookmarks, 0, "<StockItem>", ROX_STOCK_BOOKMARKS},
+{">" N_("Show Log"),		NULL, show_log, 0, "<StockItem>", GTK_STOCK_INFO},
 {">" N_("Follow Symbolic Links"),	NULL, follow_symlinks, 0, NULL},
 {">" N_("Resize Window"),	"<Ctrl>E", resize, 0, NULL},
 /* {">" N_("New, As User..."),	NULL, new_user, 0, NULL}, */
@@ -287,6 +297,7 @@ gboolean ensure_filer_menu(void)
 	GET_MENU_ITEM(filer_menu, "filer");
 	GET_SMENU_ITEM(filer_file_menu, "filer", "File");
 	GET_SSMENU_ITEM(filer_hidden_menu, "filer", "Display", "Show Hidden");
+	GET_SSMENU_ITEM(filer_filter_dirs_menu, "filer", "Display", "Filter Directories With Files");
 	GET_SSMENU_ITEM(filer_reverse_menu, "filer", "Display", "Reversed");
 	GET_SSMENU_ITEM(filer_auto_size_menu, "filer", "Display", "Automatic");
 	GET_SSMENU_ITEM(filer_thumb_menu, "filer", "Display",
@@ -736,6 +747,9 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 		gtk_check_menu_item_set_active(
 				GTK_CHECK_MENU_ITEM(filer_hidden_menu),
 				filer_window->show_hidden);
+		gtk_check_menu_item_set_active(
+				GTK_CHECK_MENU_ITEM(filer_filter_dirs_menu),
+				filer_window->filter_directories);
 		gtk_check_menu_item_set_active(
 				GTK_CHECK_MENU_ITEM(filer_reverse_menu),
 				filer_window->sort_order != GTK_SORT_ASCENDING);
@@ -963,6 +977,17 @@ static void hidden(gpointer data, guint action, GtkWidget *widget)
 			   !window_with_focus->show_hidden);
 }
 
+static void filter_directories(gpointer data, guint action, GtkWidget *widget)
+{
+	if (updating_menu)
+		return;
+	
+	g_return_if_fail(window_with_focus != NULL);
+
+	display_set_filter_directories(window_with_focus,
+			   !window_with_focus->filter_directories);
+}
+
 static void show_thumbs(gpointer data, guint action, GtkWidget *widget)
 {
 	if (updating_menu)
@@ -1044,6 +1069,8 @@ static void find(FilerWindow *filer_window)
 	destroy_glist(&paths);
 }
 
+static gboolean last_symlink_check_relative = TRUE;
+
 /* This creates a new savebox widget, and allows the user to pick a new path
  * for the file.
  * Once the new path has been picked, the callback will be called with
@@ -1067,7 +1094,7 @@ static void savebox_show(const gchar *action, const gchar *path,
 		check_relative = gtk_check_button_new_with_mnemonic(
 							_("_Relative link"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_relative),
-					     TRUE);
+					     last_symlink_check_relative);
 
 		GTK_WIDGET_UNSET_FLAGS(check_relative, GTK_CAN_FOCUS);
 		gtk_tooltips_set_tip(tooltips, check_relative,
@@ -1195,7 +1222,9 @@ static gboolean link_cb(GObject *savebox,
 
 	check_relative = g_object_get_data(savebox, "check_relative");
 
-	if (gtk_toggle_button_get_active(check_relative))
+	last_symlink_check_relative = gtk_toggle_button_get_active(check_relative);
+
+	if (last_symlink_check_relative)
 		link_path = get_relative_path(path, initial);
 	else
 		link_path = g_strdup(initial);
@@ -1384,7 +1413,7 @@ static gboolean new_file_type_cb(GObject *savebox,
 			         const gchar *initial, const gchar *path)
 {
 	const gchar *oleaf, *leaf;
-	gchar *templ, *templ_dname, *dest;
+	gchar *templ, *rtempl, *templ_dname, *dest;
 	GList *paths;
 
 	/* We can work out the template path from the initial name */
@@ -1400,16 +1429,18 @@ static gboolean new_file_type_cb(GObject *savebox,
 
 	templ = g_strconcat(templ_dname, "/", oleaf, NULL);
 	g_free(templ_dname);
+	rtempl = pathdup(templ);
+	g_free(templ);
 
 	dest = g_path_get_dirname(path);
 	leaf = g_basename(path);
-	paths = g_list_append(NULL, templ);
+	paths = g_list_append(NULL, rtempl);
 
 	action_copy(paths, dest, leaf, TRUE);
 
 	g_list_free(paths);
 	g_free(dest);
-	g_free(templ);
+	g_free(rtempl);
 
 	if (filer_exists(window_with_focus))
 		display_set_autoselect(window_with_focus, leaf);
@@ -1691,6 +1722,13 @@ static void show_bookmarks(gpointer data, guint action, GtkWidget *widget)
 	bookmarks_show_menu(window_with_focus);
 }
 
+static void show_log(gpointer data, guint action, GtkWidget *widget)
+{
+	g_return_if_fail(window_with_focus != NULL);
+
+	log_show_window();
+}
+
 static void follow_symlinks(gpointer data, guint action, GtkWidget *widget)
 {
 	g_return_if_fail(window_with_focus != NULL);
